@@ -11,14 +11,12 @@
 #include "calendarmanager.h"
 
 // Akonadi
-#include <akonadi_version.h>
 #if AKONADI_VERSION >= QT_VERSION_CHECK(5, 18, 41)
 #include <Akonadi/AgentInstanceModel>
 #include <Akonadi/AgentManager>
 #include <Akonadi/AttributeFactory>
 #include <Akonadi/CollectionColorAttribute>
 #include <Akonadi/CollectionDeleteJob>
-#include <Akonadi/CollectionFilterProxyModel>
 #include <Akonadi/CollectionIdentificationAttribute>
 #include <Akonadi/CollectionModifyJob>
 #include <Akonadi/CollectionPropertiesDialog>
@@ -47,20 +45,14 @@
 #include <Monitor>
 #include <control.h>
 #endif
+#include "kalendar_debug.h"
 #include <Akonadi/Calendar/History>
-#include <Akonadi/Calendar/IncidenceChanger>
-#include <CalendarSupport/KCalPrefs>
-#include <CalendarSupport/Utils>
 #include <EventViews/Prefs>
 #include <KCheckableProxyModel>
-#include <KDescendantsProxyModel>
-#include <KFormat>
 #include <KLocalizedString>
 #include <QApplication>
-#include <QMetaEnum>
 #include <QPointer>
 #include <QRandomGenerator>
-#include <QTimer>
 #include <etmcalendar.h>
 
 using namespace Akonadi;
@@ -226,7 +218,7 @@ public:
     QVariant data(const QModelIndex &index, int role) const override
     {
         if (!index.isValid()) {
-            return QVariant();
+            return {};
         }
         if (role == Qt::DecorationRole) {
             const Akonadi::Collection collection = CalendarSupport::collectionFromIndex(index);
@@ -301,7 +293,7 @@ public:
         }
 
         if (collection.hasAttribute<Akonadi::CollectionColorAttribute>()) {
-            const auto *colorAttr = collection.attribute<Akonadi::CollectionColorAttribute>();
+            const auto colorAttr = collection.attribute<Akonadi::CollectionColorAttribute>();
             if (colorAttr && colorAttr->color().isValid()) {
                 colorCache[id] = colorAttr->color();
                 save();
@@ -606,7 +598,7 @@ qint64 CalendarManager::defaultCalendarId(IncidenceWrapper *incidenceWrapper)
 
 int CalendarManager::getCalendarSelectableIndex(IncidenceWrapper *incidenceWrapper)
 {
-    KDescendantsProxyModel *model = new KDescendantsProxyModel;
+    auto model = new KDescendantsProxyModel;
 
     switch (incidenceWrapper->incidencePtr()->type()) {
     default:
@@ -659,6 +651,11 @@ QVariantMap CalendarManager::undoRedoData()
     };
 }
 
+Akonadi::Item CalendarManager::incidenceItem(KCalendarCore::Incidence::Ptr incidence)
+{
+    return m_calendar->item(incidence);
+}
+
 void CalendarManager::addIncidence(IncidenceWrapper *incidenceWrapper)
 {
     Akonadi::Collection collection(incidenceWrapper->collectionId());
@@ -700,13 +697,90 @@ void CalendarManager::editIncidence(IncidenceWrapper *incidenceWrapper)
 
     Akonadi::Collection newCollection(incidenceWrapper->collectionId());
     modifiedItem.setParentCollection(newCollection);
-    Akonadi::ItemMoveJob *job = new Akonadi::ItemMoveJob(modifiedItem, newCollection);
+    auto job = new Akonadi::ItemMoveJob(modifiedItem, newCollection);
     // Add some type of check here?
     connect(job, &KJob::result, job, [=]() {
-        qDebug() << job->error();
+        qCDebug(KALENDAR_LOG) << job->error();
     });
 }
 
+void CalendarManager::updateIncidenceDates(IncidenceWrapper *incidenceWrapper, int startOffset, int endOffset, int occurrences, const QDateTime &occurrenceDate)
+{ // start and end offsets are in msecs
+
+    Akonadi::Item item = m_calendar->item(incidenceWrapper->incidencePtr());
+    item.setPayload(incidenceWrapper->incidencePtr());
+
+    auto setNewDates = [&](KCalendarCore::Incidence::Ptr incidence) {
+        if (incidence->type() == KCalendarCore::Incidence::TypeTodo) {
+            // For to-dos endOffset is ignored because it will always be == to startOffset because we only
+            // support moving to-dos, not resizing them. There are no multi-day to-dos.
+            // Lets just call it offset to reduce confusion.
+            const int offset = startOffset;
+
+            KCalendarCore::Todo::Ptr todo = incidence.staticCast<KCalendarCore::Todo>();
+            QDateTime due = todo->dtDue();
+            QDateTime start = todo->dtStart();
+            if (due.isValid()) { // Due has priority over start.
+                // We will only move the due date, unlike events where we move both.
+                due = due.addMSecs(offset);
+                todo->setDtDue(due);
+
+                if (start.isValid() && start > due) {
+                    // Start can't be bigger than due.
+                    todo->setDtStart(due);
+                }
+            } else if (start.isValid()) {
+                // So we're displaying a to-do that doesn't have due date, only start...
+                start = start.addMSecs(offset);
+                todo->setDtStart(start);
+            } else {
+                // This never happens
+                // qCWarning(CALENDARVIEW_LOG) << "Move what? uid:" << todo->uid() << "; summary=" << todo->summary();
+            }
+        } else {
+            incidence->setDtStart(incidence->dtStart().addMSecs(startOffset));
+            if (incidence->type() == KCalendarCore::Incidence::TypeEvent) {
+                KCalendarCore::Event::Ptr event = incidence.staticCast<KCalendarCore::Event>();
+                event->setDtEnd(event->dtEnd().addMSecs(endOffset));
+            }
+        }
+    };
+
+    if (incidenceWrapper->incidencePtr()->recurs()) {
+        switch (occurrences) {
+        case KCalUtils::RecurrenceActions::AllOccurrences: {
+            // All occurrences
+            KCalendarCore::Incidence::Ptr oldIncidence(incidenceWrapper->incidencePtr()->clone());
+            setNewDates(incidenceWrapper->incidencePtr());
+            qCDebug(KALENDAR_LOG) << incidenceWrapper->incidenceStart();
+            m_changer->modifyIncidence(item, oldIncidence);
+            break;
+        }
+        case KCalUtils::RecurrenceActions::SelectedOccurrence: // Just this occurrence
+        case KCalUtils::RecurrenceActions::FutureOccurrences: { // All future occurrences
+            const bool thisAndFuture = (occurrences == KCalUtils::RecurrenceActions::FutureOccurrences);
+            auto tzedOccurrenceDate = occurrenceDate.toTimeZone(incidenceWrapper->incidenceStart().timeZone());
+            KCalendarCore::Incidence::Ptr newIncidence(
+                KCalendarCore::Calendar::createException(incidenceWrapper->incidencePtr(), tzedOccurrenceDate, thisAndFuture));
+
+            if (newIncidence) {
+                m_changer->startAtomicOperation(i18n("Move occurrence(s)"));
+                setNewDates(newIncidence);
+                m_changer->createIncidence(newIncidence, m_calendar->collection(incidenceWrapper->collectionId()));
+                m_changer->endAtomicOperation();
+            } else {
+                qCDebug(KALENDAR_LOG) << i18n("Unable to add the exception item to the calendar. No change will be done.");
+            }
+            break;
+        }
+        }
+    } else { // Doesn't recur
+        KCalendarCore::Incidence::Ptr oldIncidence(incidenceWrapper->incidencePtr()->clone());
+        setNewDates(incidenceWrapper->incidencePtr());
+        m_changer->modifyIncidence(item, oldIncidence);
+    }
+}
+
 bool CalendarManager::hasChildren(KCalendarCore::Incidence::Ptr incidence)
 {
     return !m_calendar->childIncidences(incidence->uid()).isEmpty();
@@ -793,12 +867,12 @@ QVariantMap CalendarManager::getCollectionDetails(QVariant collectionId)
 void CalendarManager::setCollectionColor(qint64 collectionId, QColor color)
 {
     auto collection = m_calendar->collection(collectionId);
-    Akonadi::CollectionColorAttribute *colorAttr = collection.attribute<Akonadi::CollectionColorAttribute>(Akonadi::Collection::AddIfMissing);
+    auto colorAttr = collection.attribute<Akonadi::CollectionColorAttribute>(Akonadi::Collection::AddIfMissing);
     colorAttr->setColor(color);
-    Akonadi::CollectionModifyJob *modifyJob = new Akonadi::CollectionModifyJob(collection);
+    auto modifyJob = new Akonadi::CollectionModifyJob(collection);
     connect(modifyJob, &Akonadi::CollectionModifyJob::result, this, [this, collectionId, color](KJob *job) {
         if (job->error()) {
-            qWarning() << "Error occurred modifying collection color: " << job->errorString();
+            qCWarning(KALENDAR_LOG) << "Error occurred modifying collection color: " << job->errorString();
         } else {
             m_baseModel->colorCache[QString::number(collectionId)] = color;
             m_baseModel->save();
@@ -816,6 +890,14 @@ void CalendarManager::redoAction()
     m_changer->history()->redo();
 }
 
+void CalendarManager::updateAllCollections()
+{
+    for (int i = 0; i < collections()->rowCount(); i++) {
+        auto collection = collections()->data(collections()->index(i, 0), Akonadi::EntityTreeModel::CollectionRole).value<Akonadi::Collection>();
+        Akonadi::AgentManager::self()->synchronizeCollection(collection, true);
+    }
+}
+
 void CalendarManager::updateCollection(qint64 collectionId)
 {
     auto collection = m_calendar->collection(collectionId);
@@ -832,7 +914,7 @@ void CalendarManager::deleteCollection(qint64 collectionId)
         auto job = new Akonadi::CollectionDeleteJob(collection, this);
         connect(job, &Akonadi::CollectionDeleteJob::result, this, [this](KJob *job) {
             if (job->error()) {
-                qWarning() << "Error occurred deleting collection: " << job->errorString();
+                qCWarning(KALENDAR_LOG) << "Error occurred deleting collection: " << job->errorString();
             }
         });
         return;
