@@ -12,14 +12,12 @@
 #include <QColor>
 #include <QDateTime>
 #include <QList>
-#include <QSet>
 #include <QSharedPointer>
 #include <QTimer>
 #include <etmcalendar.h>
 
 namespace KCalendarCore
 {
-class MemoryCalendar;
 class Incidence;
 }
 namespace Akonadi
