@@ -8,7 +8,7 @@
 #include "kalendarconfig.h"
 #include <Akonadi/Calendar/ETMCalendar>
 #include <Akonadi/Calendar/ICalImporter>
-#include <KXmlGui/KActionCollection>
+#include <KActionCollection>
 #include <QActionGroup>
 #include <QObject>
 
@@ -77,6 +77,7 @@ Q_SIGNALS:
     void importIntoExistingFinished(bool success, int total);
     void importIntoNewFinished(bool success);
     void importErrorMessageChanged();
+    void refreshAllCalendars();
 
 private:
     KActionCollection mCollection;
